@@ -50,6 +50,18 @@
 /* Number of scratch surfaces beyond those used as reference */
 #define SCRATCH_SURFACES_COUNT (4)
 
+static gboolean
+ensure_formats (GstVaapiContext * context)
+{
+  if (G_LIKELY (context->formats))
+    return TRUE;
+
+  context->formats =
+      gst_vaapi_get_surface_formats (GST_VAAPI_OBJECT_DISPLAY (context),
+      context->va_config);
+  return (context->formats != NULL);
+}
+
 static void
 unref_surface_cb (GstVaapiSurface * surface)
 {
@@ -106,6 +118,11 @@ context_destroy (GstVaapiContext * context)
       GST_WARNING ("failed to destroy config 0x%08x", context->va_config);
     context->va_config = VA_INVALID_ID;
   }
+
+  if (context->formats) {
+    g_array_unref (context->formats);
+    context->formats = NULL;
+  }
 }
 
 static gboolean
@@ -164,13 +181,12 @@ context_create (GstVaapiContext * context)
 {
   const GstVaapiContextInfo *const cip = &context->info;
   GstVaapiDisplay *const display = GST_VAAPI_OBJECT_DISPLAY (context);
-  VAConfigAttrib attribs[3], *attrib = attribs;
   VAContextID context_id;
   VASurfaceID surface_id;
   VAStatus status;
   GArray *surfaces = NULL;
   gboolean success = FALSE;
-  guint i, value, va_chroma_format;
+  guint i;
 
   if (!context->surfaces && !context_create_surfaces (context))
     goto cleanup;
@@ -190,6 +206,33 @@ context_create (GstVaapiContext * context)
   }
   g_assert (surfaces->len == context->surfaces->len);
 
+  GST_VAAPI_DISPLAY_LOCK (display);
+  status = vaCreateContext (GST_VAAPI_DISPLAY_VADISPLAY (display),
+      context->va_config, cip->width, cip->height, VA_PROGRESSIVE,
+      (VASurfaceID *) surfaces->data, surfaces->len, &context_id);
+  GST_VAAPI_DISPLAY_UNLOCK (display);
+  if (!vaapi_check_status (status, "vaCreateContext()"))
+    goto cleanup;
+
+  GST_DEBUG ("context 0x%08x", context_id);
+  GST_VAAPI_OBJECT_ID (context) = context_id;
+  success = TRUE;
+
+cleanup:
+  if (surfaces)
+    g_array_free (surfaces, TRUE);
+  return success;
+}
+
+static gboolean
+config_create (GstVaapiContext * context)
+{
+  const GstVaapiContextInfo *const cip = &context->info;
+  GstVaapiDisplay *const display = GST_VAAPI_OBJECT_DISPLAY (context);
+  VAConfigAttrib attribs[3], *attrib = attribs;
+  VAStatus status;
+  guint value, va_chroma_format;
+
   /* Reset profile and entrypoint */
   if (!cip->profile || !cip->entrypoint)
     goto cleanup;
@@ -272,22 +315,10 @@ context_create (GstVaapiContext * context)
   if (!vaapi_check_status (status, "vaCreateConfig()"))
     goto cleanup;
 
-  GST_VAAPI_DISPLAY_LOCK (display);
-  status = vaCreateContext (GST_VAAPI_DISPLAY_VADISPLAY (display),
-      context->va_config, cip->width, cip->height, VA_PROGRESSIVE,
-      (VASurfaceID *) surfaces->data, surfaces->len, &context_id);
-  GST_VAAPI_DISPLAY_UNLOCK (display);
-  if (!vaapi_check_status (status, "vaCreateContext()"))
-    goto cleanup;
-
-  GST_DEBUG ("context 0x%08x", context_id);
-  GST_VAAPI_OBJECT_ID (context) = context_id;
-  success = TRUE;
-
+  return TRUE;
 cleanup:
-  if (surfaces)
-    g_array_free (surfaces, TRUE);
-  return success;
+  GST_WARNING ("Failed to create vaConfig");
+  return FALSE;
 }
 
 /** Updates config for encoding. Returns %TRUE if config changed */
@@ -325,6 +356,8 @@ gst_vaapi_context_init (GstVaapiContext * context,
   context->va_config = VA_INVALID_ID;
   context->reset_on_resize = TRUE;
   gst_vaapi_context_overlay_init (context);
+
+  context->formats = NULL;
 }
 
 static void
@@ -356,21 +389,36 @@ gst_vaapi_context_new (GstVaapiDisplay * display,
 
   g_return_val_if_fail (cip->profile, NULL);
   g_return_val_if_fail (cip->entrypoint, NULL);
-  g_return_val_if_fail (cip->width > 0, NULL);
-  g_return_val_if_fail (cip->height > 0, NULL);
 
   context = gst_vaapi_object_new (gst_vaapi_context_class (), display);
   if (!context)
     return NULL;
 
   gst_vaapi_context_init (context, cip);
+
+  if (!config_create (context))
+    goto error;
+
+  /* this means we don't want to create a VAcontext */
+  if (cip->width == 0 && cip->height == 0)
+    goto done;
+
+  /* this is not valid */
+  if (cip->width == 0 || cip->height == 0)
+    goto error;
+
   if (!context_create (context))
     goto error;
+
+done:
   return context;
 
+  /* ERRORS */
 error:
-  gst_vaapi_object_unref (context);
-  return NULL;
+  {
+    gst_vaapi_object_unref (context);
+    return NULL;
+  }
 }
 
 /**
@@ -439,7 +487,7 @@ gst_vaapi_context_reset (GstVaapiContext * context,
     return FALSE;
   else if (grow_surfaces && !context_ensure_surfaces (context))
     return FALSE;
-  if (reset_config && !context_create (context))
+  if (reset_config && !(config_create (context) && context_create (context)))
     return FALSE;
   return TRUE;
 }
@@ -517,3 +565,25 @@ gst_vaapi_context_reset_on_resize (GstVaapiContext * context,
 
   context->reset_on_resize = reset_on_resize;
 }
+
+/**
+ * gst_vaapi_context_get_surface_formats:
+ * @context: a #GstVaapiContext
+ *
+ * Determines the set of supported formats by the surfaces associated
+ * to @context. The caller owns an extra reference of the resulting
+ * array of #GstVideoFormat elements, so it shall be released with
+ * g_array_unref after usage.
+ *
+ * Return value: (transfer full): the set of target formats supported
+ * by the surfaces in @context.
+ */
+GArray *
+gst_vaapi_context_get_surface_formats (GstVaapiContext * context)
+{
+  g_return_val_if_fail (context, NULL);
+
+  if (!ensure_formats (context))
+    return NULL;
+  return g_array_ref (context->formats);
+}
