@@ -40,6 +40,26 @@
 
 static void drop_frame (GstVaapiDecoder * decoder, GstVideoCodecFrame * frame);
 
+static void
+parser_state_reset (GstVaapiParserState * ps)
+{
+
+  if (ps->input_adapter)
+    gst_adapter_clear (ps->input_adapter);
+  if (ps->output_adapter)
+    gst_adapter_clear (ps->output_adapter);
+  ps->current_adapter = NULL;
+
+  if (ps->next_unit_pending) {
+    gst_vaapi_decoder_unit_clear (&ps->next_unit);
+    ps->next_unit_pending = FALSE;
+  }
+
+  ps->current_frame_number = 0;
+  ps->input_offset1 = ps->input_offset2 = 0;
+  ps->at_eos = FALSE;
+}
+
 static void
 parser_state_finalize (GstVaapiParserState * ps)
 {
@@ -266,16 +286,6 @@ do_decode (GstVaapiDecoder * decoder, GstVideoCodecFrame * base_frame)
   return status;
 }
 
-static inline GstVaapiDecoderStatus
-do_flush (GstVaapiDecoder * decoder)
-{
-  GstVaapiDecoderClass *const klass = GST_VAAPI_DECODER_GET_CLASS (decoder);
-
-  if (klass->flush)
-    return klass->flush (decoder);
-  return GST_VAAPI_DECODER_STATUS_SUCCESS;
-}
-
 static GstVaapiDecoderStatus
 decode_step (GstVaapiDecoder * decoder)
 {
@@ -418,6 +428,8 @@ set_caps (GstVaapiDecoder * decoder, const GstCaps * caps)
   if (!gst_video_info_from_caps (&codec_state->info, caps))
     return FALSE;
 
+  if (codec_state->caps)
+    gst_caps_unref (codec_state->caps);
   codec_state->caps = gst_caps_copy (caps);
 
   v_codec_data = gst_structure_get_value (structure, "codec_data");
@@ -531,9 +543,12 @@ gst_vaapi_decoder_new (const GstVaapiDecoderClass * klass,
     goto error;
   return decoder;
 
+  /* ERRORS */
 error:
-  gst_vaapi_decoder_unref (decoder);
-  return NULL;
+  {
+    gst_vaapi_decoder_unref (decoder);
+    return NULL;
+  }
 }
 
 /**
@@ -1024,13 +1039,69 @@ gst_vaapi_decoder_decode (GstVaapiDecoder * decoder, GstVideoCodecFrame * frame)
   return do_decode (decoder, frame);
 }
 
+/* This function really marks the end of input,
+ * so that the decoder will drain out any pending
+ * frames on calls to gst_vaapi_decoder_get_frame_with_timeout() */
 GstVaapiDecoderStatus
 gst_vaapi_decoder_flush (GstVaapiDecoder * decoder)
 {
+  GstVaapiDecoderClass *klass;
+
   g_return_val_if_fail (decoder != NULL,
       GST_VAAPI_DECODER_STATUS_ERROR_INVALID_PARAMETER);
 
-  return do_flush (decoder);
+  klass = GST_VAAPI_DECODER_GET_CLASS (decoder);
+
+  if (klass->flush)
+    return klass->flush (decoder);
+
+  return GST_VAAPI_DECODER_STATUS_SUCCESS;
+}
+
+/* Reset the decoder instance to a clean state,
+ * clearing any pending decode state, without
+ * reallocating the entire decoder */
+GstVaapiDecoderStatus
+gst_vaapi_decoder_reset (GstVaapiDecoder * decoder)
+{
+  GstVaapiDecoderClass *klass;
+  GstVaapiDecoderStatus ret = GST_VAAPI_DECODER_STATUS_SUCCESS;
+
+  g_return_val_if_fail (decoder != NULL,
+      GST_VAAPI_DECODER_STATUS_ERROR_INVALID_PARAMETER);
+
+  klass = GST_VAAPI_DECODER_GET_CLASS (decoder);
+
+  GST_DEBUG ("Resetting decoder");
+
+  if (klass->reset) {
+    ret = klass->reset (decoder);
+  } else {
+    if (klass->destroy)
+      klass->destroy (decoder);
+    if (klass->create)
+      if (!klass->create (decoder))
+        ret = GST_VAAPI_DECODER_STATUS_ERROR_UNKNOWN;
+  }
+
+  if (ret != GST_VAAPI_DECODER_STATUS_SUCCESS)
+    return ret;
+
+  /* Clear any buffers and frame in the queues */
+  {
+    GstVideoCodecFrame *frame;
+    GstBuffer *buffer;
+
+    while ((frame = g_async_queue_try_pop (decoder->frames)) != NULL)
+      gst_video_codec_frame_unref (frame);
+
+    while ((buffer = g_async_queue_try_pop (decoder->buffers)) != NULL)
+      gst_buffer_unref (buffer);
+  }
+
+  parser_state_reset (&decoder->parser_state);
+
+  return GST_VAAPI_DECODER_STATUS_SUCCESS;
 }
 
 GstVaapiDecoderStatus
@@ -1064,3 +1135,68 @@ gst_vaapi_decoder_decode_codec_data (GstVaapiDecoder * decoder)
   gst_buffer_unmap (codec_data, &map_info);
   return status;
 }
+
+/**
+ * gst_vaapi_decoder_get_surface_formats:
+ * @decoder: a #GstVaapiDecoder
+ *
+ * Retrieves an array of #GstVideoFormat which the output surfaces of
+ * the @decoder can handle.
+ *
+ * Return value: (transfer full): a #GArray of #GstVideoFormat or
+ * %NULL
+ */
+GArray *
+gst_vaapi_decoder_get_surface_formats (GstVaapiDecoder * decoder)
+{
+  if (decoder && decoder->context)
+    return gst_vaapi_context_get_surface_formats (decoder->context);
+  return NULL;
+}
+
+/**
+ * gst_vaapi_decoder_update_caps:
+ * @decoder: a #GstVaapiDecoder
+ * @caps: a #GstCaps
+ *
+ * If @caps is compatible with the current caps, or they have the same
+ * codec, the caps are updated internally.
+ *
+ * This method will not call codec_state_changed() callback, since
+ * this function is intended to run sync and during the set_format()
+ * vmethod.
+ *
+ * Returns: %TRUE if the caps were updated internally.
+ **/
+gboolean
+gst_vaapi_decoder_update_caps (GstVaapiDecoder * decoder, GstCaps * caps)
+{
+  GstCaps *decoder_caps;
+  GstVaapiProfile profile;
+  GstVaapiCodec codec;
+
+  g_return_val_if_fail (decoder != NULL, FALSE);
+  g_return_val_if_fail (caps != NULL, FALSE);
+
+  decoder_caps = get_caps (decoder);
+  if (!decoder_caps)
+    return FALSE;
+
+  if (gst_caps_is_always_compatible (caps, decoder_caps)) {
+    set_caps (decoder, caps);
+    return TRUE;
+  }
+
+  profile = gst_vaapi_profile_from_caps (caps);
+  if (profile == GST_VAAPI_PROFILE_UNKNOWN)
+    return FALSE;
+  codec = gst_vaapi_profile_get_codec (profile);
+  if (codec == 0)
+    return FALSE;
+  if (codec == decoder->codec) {
+    set_caps (decoder, caps);
+    return TRUE;
+  }
+
+  return FALSE;
+}
