@@ -39,7 +39,8 @@
 
 /* Supported set of VA rate controls, within this implementation */
 #define SUPPORTED_RATECONTROLS                  \
-  (GST_VAAPI_RATECONTROL_MASK (CQP))
+  (GST_VAAPI_RATECONTROL_MASK (CQP) |           \
+   GST_VAAPI_RATECONTROL_MASK (CBR))
 
 /* Supported set of tuning options, within this implementation */
 #define SUPPORTED_TUNE_OPTIONS \
@@ -129,8 +130,9 @@ ensure_bitrate (GstVaapiEncoderVP8 * encoder)
             gst_util_uint64_scale (GST_VAAPI_ENCODER_WIDTH (encoder) *
             GST_VAAPI_ENCODER_HEIGHT (encoder),
             GST_VAAPI_ENCODER_FPS_N (encoder),
-            GST_VAAPI_ENCODER_FPS_D (encoder)) / 4 * 1000;
+            GST_VAAPI_ENCODER_FPS_D (encoder)) / (4 * 1000);
       }
+      break;
     default:
       base_encoder->bitrate = 0;
       break;
@@ -219,7 +221,7 @@ fill_sequence (GstVaapiEncoderVP8 * encoder, GstVaapiEncSequence * sequence)
   seq_param->frame_height = GST_VAAPI_ENCODER_HEIGHT (encoder);
 
   if (GST_VAAPI_ENCODER_RATE_CONTROL (encoder) & GST_VAAPI_RATECONTROL_CBR)
-    seq_param->bits_per_second = base_encoder->bitrate;
+    seq_param->bits_per_second = base_encoder->bitrate * 1000;
 
   seq_param->intra_period = base_encoder->keyframe_period;
 
@@ -247,9 +249,71 @@ ensure_sequence (GstVaapiEncoderVP8 * encoder, GstVaapiEncPicture * picture)
   gst_vaapi_codec_object_replace (&sequence, NULL);
   return TRUE;
 
+  /* ERRORS */
 error:
-  gst_vaapi_codec_object_replace (&sequence, NULL);
-  return FALSE;
+  {
+    gst_vaapi_codec_object_replace (&sequence, NULL);
+    return FALSE;
+  }
+}
+
+static gboolean
+ensure_misc_params (GstVaapiEncoderVP8 * encoder, GstVaapiEncPicture * picture)
+{
+  GstVaapiEncoder *const base_encoder = GST_VAAPI_ENCODER_CAST (encoder);
+  GstVaapiEncMiscParam *misc;
+
+  if (GST_VAAPI_ENCODER_RATE_CONTROL (encoder) != GST_VAAPI_RATECONTROL_CBR)
+    return TRUE;
+
+  misc = GST_VAAPI_ENC_MISC_PARAM_NEW (FrameRate, encoder);
+  if (!misc)
+    return FALSE;
+  {
+    VAEncMiscParameterFrameRate *param = misc->data;
+    param->framerate =
+        GST_VAAPI_ENCODER_FPS_N (encoder) / GST_VAAPI_ENCODER_FPS_D (encoder);
+  }
+  gst_vaapi_enc_picture_add_misc_param (picture, misc);
+  gst_vaapi_codec_object_replace (&misc, NULL);
+
+  misc = GST_VAAPI_ENC_MISC_PARAM_NEW (HRD, encoder);
+  if (!misc)
+    return FALSE;
+  {
+    VAEncMiscParameterHRD *hrd = misc->data;
+    if (base_encoder->bitrate > 0) {
+      hrd->buffer_size = base_encoder->bitrate * 1000 * 2;
+      hrd->initial_buffer_fullness = base_encoder->bitrate * 1000;
+    } else {
+      hrd->buffer_size = 0;
+      hrd->initial_buffer_fullness = 0;
+    }
+  }
+
+  gst_vaapi_enc_picture_add_misc_param (picture, misc);
+  gst_vaapi_codec_object_replace (&misc, NULL);
+
+  /* RateControl params */
+  misc = GST_VAAPI_ENC_MISC_PARAM_NEW (RateControl, encoder);
+  if (!misc)
+    return FALSE;
+  {
+    VAEncMiscParameterRateControl *rate_control;
+    rate_control = misc->data;
+    rate_control->bits_per_second = base_encoder->bitrate * 1000;
+    rate_control->target_percentage = 70;
+    /* CPB (Coded picture buffer) length in milliseconds, which could
+     * be provided as a property */
+    rate_control->window_size = 500;
+    rate_control->initial_qp = encoder->yac_qi;
+    rate_control->min_qp = 1;
+  }
+
+  gst_vaapi_enc_picture_add_misc_param (picture, misc);
+  gst_vaapi_codec_object_replace (&misc, NULL);
+
+  return TRUE;
 }
 
 static gboolean
@@ -299,6 +363,10 @@ fill_picture (GstVaapiEncoderVP8 * encoder,
 
   pic_param->sharpness_level = encoder->sharpness_level;
 
+  /* Used for CBR */
+  pic_param->clamp_qindex_low = 0;
+  pic_param->clamp_qindex_high = 127;
+
   return TRUE;
 }
 
@@ -371,6 +439,8 @@ gst_vaapi_encoder_vp8_encode (GstVaapiEncoder * base_encoder,
 
   if (!ensure_sequence (encoder, picture))
     goto error;
+  if (!ensure_misc_params (encoder, picture))
+    goto error;
   if (!ensure_picture (encoder, picture, codedbuf, reconstruct))
     goto error;
   if (!ensure_quantization_table (encoder, picture))
@@ -384,11 +454,15 @@ gst_vaapi_encoder_vp8_encode (GstVaapiEncoder * base_encoder,
   }
 
   return GST_VAAPI_ENCODER_STATUS_SUCCESS;
+
+  /* ERRORS */
 error:
-  if (reconstruct)
-    gst_vaapi_encoder_release_surface (GST_VAAPI_ENCODER (encoder),
-        reconstruct);
-  return ret;
+  {
+    if (reconstruct)
+      gst_vaapi_encoder_release_surface (GST_VAAPI_ENCODER (encoder),
+          reconstruct);
+    return ret;
+  }
 }
 
 static GstVaapiEncoderStatus
@@ -446,8 +520,11 @@ gst_vaapi_encoder_vp8_reconfigure (GstVaapiEncoder * base_encoder)
 
   return set_context_info (base_encoder);
 
+  /* ERRORS */
 error:
-  return GST_VAAPI_ENCODER_STATUS_ERROR_OPERATION_FAILED;
+  {
+    return GST_VAAPI_ENCODER_STATUS_ERROR_OPERATION_FAILED;
+  }
 }
 
 static gboolean
