@@ -29,7 +29,7 @@
  * <refsect2>
  * <title>Example launch line</title>
  * |[
- *  gst-launch-1.0 -ev videotestsrc num-buffers=60 ! timeoverlay ! vaapih265enc ! matroskamux ! filesink location=test.mkv
+ *  gst-launch-1.0 -ev videotestsrc num-buffers=60 ! timeoverlay ! vaapih265enc ! h265parse ! matroskamux ! filesink location=test.mkv
  * ]|
  * </refsect2>
  */
@@ -64,7 +64,7 @@ static const char gst_vaapiencode_h265_sink_caps_str[] =
 /* *INDENT-OFF* */
 static const char gst_vaapiencode_h265_src_caps_str[] =
   GST_CODEC_CAPS ", "
-  "profile = (string) { main }";
+  "profile = (string) { main, main-10 }";
 /* *INDENT-ON* */
 
 /* *INDENT-OFF* */
@@ -128,6 +128,25 @@ gst_vaapiencode_h265_get_property (GObject * object,
   }
 }
 
+static GstVaapiProfile
+gst_vaapiencode_h265_get_profile (GstCaps * caps)
+{
+  guint i;
+
+  for (i = 0; i < gst_caps_get_size (caps); i++) {
+    GstStructure *const structure = gst_caps_get_structure (caps, i);
+    const GValue *const value = gst_structure_get_value (structure, "profile");
+
+    if (value && G_VALUE_HOLDS_STRING (value)) {
+      const gchar *str = g_value_get_string (value);
+      if (str)
+        return gst_vaapi_utils_h265_get_profile_from_string (str);
+    }
+  }
+
+  return GST_VAAPI_PROFILE_UNKNOWN;
+}
+
 typedef struct
 {
   GstVaapiProfile best_profile;
@@ -339,6 +358,7 @@ _h265_convert_byte_stream_to_hvc (GstBuffer * buf)
   gst_buffer_unmap (buf, &info);
   return TRUE;
 
+  /* ERRORS */
 error:
   {
     gst_buffer_unmap (buf, &info);
@@ -395,6 +415,7 @@ gst_vaapiencode_h265_class_init (GstVaapiEncodeH265Class * klass)
   object_class->get_property = gst_vaapiencode_h265_get_property;
 
   encode_class->get_properties = gst_vaapi_encoder_h265_get_default_properties;
+  encode_class->get_profile = gst_vaapiencode_h265_get_profile;
   encode_class->set_config = gst_vaapiencode_h265_set_config;
   encode_class->get_caps = gst_vaapiencode_h265_get_caps;
   encode_class->alloc_encoder = gst_vaapiencode_h265_alloc_encoder;
