@@ -177,12 +177,12 @@ gst_vaapi_create_display_from_gl_context (GstObject * gl_context_object)
           display_type = GST_VAAPI_DISPLAY_TYPE_WAYLAND;
 #endif
       } else {
-#if USE_X11
-        if (!display_type && GST_GL_HAVE_WINDOW_X11)
+#if USE_X11 && GST_GL_HAVE_WINDOW_X11
+        if (!display_type)
           display_type = GST_VAAPI_DISPLAY_TYPE_X11;
 #endif
-#if USE_WAYLAND
-        if (!display_type && GST_GL_HAVE_WINDOW_WAYLAND)
+#if USE_WAYLAND && GST_GL_HAVE_WINDOW_WAYLAND
+        if (!display_type)
           display_type = GST_VAAPI_DISPLAY_TYPE_WAYLAND;
 #endif
       }
@@ -241,11 +241,34 @@ gst_vaapi_create_display_from_gl_context (GstObject * gl_context_object)
   return NULL;
 }
 
+static void
+gst_vaapi_find_gl_context (GstElement * element)
+{
+  GstObject *gl_context;
+  GstVaapiPluginBase *const plugin = GST_VAAPI_PLUGIN_BASE (element);
+
+  /* if the element is vaapisink or any vaapi encoder it doesn't need
+   * to know a GstGLContext in order to create an appropriate
+   * GstVaapiDisplay. Let's them to choose their own
+   * GstVaapiDisplay */
+  if (GST_IS_VIDEO_SINK (element) || GST_IS_VIDEO_ENCODER (element))
+    return;
+
+  gl_context = NULL;
+  if (!gst_vaapi_find_gl_local_context (element, &gl_context))
+    gl_context = gst_vaapi_plugin_base_create_gl_context (plugin);
+
+  if (gl_context) {
+    gst_vaapi_plugin_base_set_gl_context (plugin, gl_context);
+    gst_object_unref (gl_context);
+  }
+}
+
 gboolean
 gst_vaapi_ensure_display (GstElement * element, GstVaapiDisplayType type)
 {
   GstVaapiPluginBase *const plugin = GST_VAAPI_PLUGIN_BASE (element);
-  GstVaapiDisplay *display;
+  GstVaapiDisplay *display = NULL;
 
   g_return_val_if_fail (GST_IS_ELEMENT (element), FALSE);
 
@@ -255,10 +278,21 @@ gst_vaapi_ensure_display (GstElement * element, GstVaapiDisplayType type)
       return TRUE;
   }
 
+  /* Query for a local GstGL context. If it's found, it will be used
+   * to create the VA display */
+  if (!plugin->gl_context)
+    gst_vaapi_find_gl_context (element);
+
   /* If no neighboor, or application not interested, use system default */
-  if (plugin->gl_context)
+  if (plugin->gl_context) {
     display = gst_vaapi_create_display_from_gl_context (plugin->gl_context);
-  else
+    /* Cannot instantiate VA display based on GL context. Reset the
+     *  requested display type to ANY to try again */
+    if (!display)
+      gst_vaapi_plugin_base_set_display_type (plugin,
+          GST_VAAPI_DISPLAY_TYPE_ANY);
+  }
+  if (!display)
     display = gst_vaapi_create_display (type, plugin->display_name);
   if (!display)
     return FALSE;
@@ -269,14 +303,25 @@ gst_vaapi_ensure_display (GstElement * element, GstVaapiDisplayType type)
 }
 
 gboolean
-gst_vaapi_handle_context_query (GstQuery * query, GstVaapiDisplay * display)
+gst_vaapi_handle_context_query (GstElement * element, GstQuery * query)
 {
+  GstVaapiPluginBase *const plugin = GST_VAAPI_PLUGIN_BASE (element);
   const gchar *type = NULL;
   GstContext *context, *old_context;
 
   g_return_val_if_fail (query != NULL, FALSE);
 
-  if (!display)
+#if USE_GST_GL_HELPERS
+  if (plugin->gl_display && plugin->gl_context && plugin->gl_other_context) {
+    if (gst_gl_handle_context_query (element, query,
+            (GstGLDisplay *) plugin->gl_display,
+            (GstGLContext *) plugin->gl_context,
+            (GstGLContext *) plugin->gl_other_context))
+      return TRUE;
+  }
+#endif
+
+  if (!plugin->display)
     return FALSE;
 
   if (!gst_query_parse_context_type (query, &type))
@@ -288,9 +333,9 @@ gst_vaapi_handle_context_query (GstQuery * query, GstVaapiDisplay * display)
   gst_query_parse_context (query, &old_context);
   if (old_context) {
     context = gst_context_copy (old_context);
-    gst_vaapi_video_context_set_display (context, display);
+    gst_vaapi_video_context_set_display (context, plugin->display);
   } else {
-    context = gst_vaapi_video_context_new_with_display (display, FALSE);
+    context = gst_vaapi_video_context_new_with_display (plugin->display, FALSE);
   }
 
   gst_query_set_context (query, context);
@@ -661,6 +706,21 @@ gst_caps_has_vaapi_surface (GstCaps * caps)
   return _gst_caps_has_feature (caps, GST_CAPS_FEATURE_MEMORY_VAAPI_SURFACE);
 }
 
+gboolean
+gst_caps_is_video_raw (GstCaps * caps)
+{
+  GstStructure *structure;
+
+  g_return_val_if_fail (caps != NULL, FALSE);
+
+  if (!gst_caps_is_fixed (caps))
+    return FALSE;
+  if (!_gst_caps_has_feature (caps, GST_CAPS_FEATURE_MEMORY_SYSTEM_MEMORY))
+    return FALSE;
+  structure = gst_caps_get_structure (caps, 0);
+  return gst_structure_has_name (structure, "video/x-raw");
+}
+
 void
 gst_video_info_change_format (GstVideoInfo * vip, GstVideoFormat format,
     guint width, guint height)
