@@ -48,6 +48,7 @@ _transform_format (GstVaapiPostproc * postproc, GstCapsFeatures * features,
     return;
 
   gst_structure_set_value (structure, "format", &value);
+  g_value_unset (&value);
 }
 
 static void
@@ -175,6 +176,8 @@ _fixate_frame_size (GstVaapiPostproc * postproc, GstVideoInfo * vinfo,
     gint num, den;
 
     from_par_n = GST_VIDEO_INFO_PAR_N (vinfo);
+    if (from_par_n == 0)
+      from_par_n = 1;
     from_par_d = GST_VIDEO_INFO_PAR_D (vinfo);
     from_w = GST_VIDEO_INFO_WIDTH (vinfo);
     from_h = GST_VIDEO_INFO_HEIGHT (vinfo);
@@ -615,11 +618,11 @@ _set_preferred_format (GstStructure * outs, GstVideoFormat format)
   if (format == GST_VIDEO_FORMAT_UNKNOWN || format == GST_VIDEO_FORMAT_ENCODED)
     return FALSE;
 
-  if (gst_vaapi_value_set_format (&value, format)) {
-    gst_structure_set_value (outs, "format", &value);
-    return TRUE;
-  }
-  return FALSE;
+  if (!gst_vaapi_value_set_format (&value, format))
+    return FALSE;
+  gst_structure_set_value (outs, "format", &value);
+  g_value_unset (&value);
+  return TRUE;
 }
 
 static GstCaps *
